#!/usr/bin/env python

import sys
sys.path.append("../lib")

from pyworks.web import Server, RequestHandler, Resource, HTTPError
from pyworks.svn import SvnRepository, SvnNode

PORT = 8001

if len(sys.argv) != 2:
    print "svn-browse.py <path-to-repository>"
    sys.exit(1)

repos = SvnRepository(sys.argv[1])

class SvnRequestHandler(RequestHandler):
    
    server_version = "SVN Demo"
    
    def get_resource(self, request):
        
        path = request.full_path
        node = repos.get_node(request.full_path)

        kind = node.node_kind()
        
        if kind == SvnNode.NONE:
            raise HTTPError(404, "Resource Not Found")
        elif kind == SvnNode.FILE:
            return SvnFileNodeResource(node)
        elif kind == SvnNode.DIR:
            return SvnDirNodeResource(node)
        else:
            raise HTTPError(500, "Unknown node type")


DIR_TEMPLATE = """
<html><head><title>Revision %(rev)s: %(path)s</title></head>
<body>
 <h2>Revision %(rev)s: %(path)s</h2>
 <ul>
  %(entry_list)s
 </ul>
 <hr noshade><em>This is a sample of pyworks.web and pyworks.svn</em>
</body></html>
"""

LI_TEMPLATE = """
<li><a href="%(href)s">%(text)s</a></li>
"""

class SvnDirNodeResource(Resource):
    
    def __init__(self, node):
        self.node = node

    def content(self):
        rev = repos.latest_rev()
        path = self.node.path
        
        entry_list = LI_TEMPLATE % {"href": "../", "text": ".."}
        for entry in self.node.get_entries():
            node = repos.get_node(path + "/" + entry)
            if node.node_kind() == SvnNode.DIR:
                href = entry + "/"
            else:
                href = entry
            text = href
            entry_list += LI_TEMPLATE % locals()
            
        return "text/html", DIR_TEMPLATE % locals()


class SvnFileNodeResource(Resource):

    def __init__(self, node):
        self.node = node

    def content(self):
        file_content = self.node.get_stream().read()
        # @@@ should look up mime type property
        return "text/plain", file_content


print "Listening on port %s..." % PORT
Server("", PORT, SvnRequestHandler).serve_forever()
