#!/usr/bin/env python

import httplib, socket, urllib

import sys
sys.path.append("../lib")
from app import parse_introspection
from atom import parse_atom


SERVER = "localhost:8000"

def urlsplit(url):
    return urllib.splithost(urllib.splittype(url)[1])


def request(server, method, path, body=None, headers=None):
    conn = httplib.HTTPConnection(server)
    try:
        print "trying %s %s to %s" % (method, path, server)
        if body is not None and headers is not None:
            conn.request(method, path, body, headers)
        else:
            conn.request(method, path)
        resp = conn.getresponse()
        conn.close()
        print "got %s %s" % (resp.status, resp.reason)
        return resp
    except socket.error:
        print "got socket error"
        return None

## Retrieving an Introspection Document (APP-07 5.1)
    
print "getting introspection document..."

collection_href = None
    
response = request(SERVER, "GET", "/introspection")
if response:
    assert response.status == 200
    doc = response.read()
    service = parse_introspection(doc)
    print "service:"
    for workspace in service.workspaces:
        print "\tworkspace:", workspace.title
        for collection in workspace.collections:
            print "\t\tcollection:", collection.title
            print "\t\t\thref =", collection.href
            print "\t\t\tmember-type =", collection.member_type
    collection1_href = service.workspaces[0].collections[0].href
    collection2_href = service.workspaces[0].collections[1].href
    collection3_href = service.workspaces[0].collections[2].href

raw_input("press return")

## List third collection

if collection3_href:
    print "getting third collection..."

    server, path = urlsplit(collection3_href)
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for link in collection.links:
            print "\t\tlink %s: %s" % (link.rel, link.href)
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        next_href = collection.links[0].href

raw_input("press return")

if collection3_href:
    print "getting third collection with wrong param..."

    server, path = urlsplit(collection3_href) 
     
    path += "?foo=bar"
    
    response = request(server, "GET", path)
    if response:
        assert response.status == 400

raw_input("press return")

if collection3_href:
    print "getting third collection with extra param..."

    server, path = urlsplit(collection3_href) 
     
    path += "?index=5&foo=bar"
    
    response = request(server, "GET", path)
    if response:
        assert response.status == 400

raw_input("press return")

if collection3_href:
    print "getting third collection with two indexes..."

    server, path = urlsplit(collection3_href) 
     
    path += "?index=5&index=3r"
    
    response = request(server, "GET", path)
    if response:
        assert response.status == 400

raw_input("press return")

if collection3_href:
    print "getting third collection with negative index..."

    server, path = urlsplit(collection3_href) 
     
    path += "?index=-5"
    
    response = request(server, "GET", path)
    if response:
        assert response.status == 400

raw_input("press return")

if collection3_href:
    print "getting third collection with index too high..."

    server, path = urlsplit(collection3_href) 
     
    path += "?index=1000"
    
    response = request(server, "GET", path)
    if response:
        assert response.status == 400

raw_input("press return")

if collection3_href:
    print "getting third collection with string index..."

    server, path = urlsplit(collection3_href) 
     
    path += "?index=foo"
    
    response = request(server, "GET", path)
    if response:
        assert response.status == 400

raw_input("press return")